;;; large-file-warning-suppress.el --- suppress some large file warnings

;; Copyright 2014 Kevin Ryde

;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Version: 0
;; Keywords: files
;; URL: http://user42.tuxfamily.org/large-file-warning-suppress/index.html
;;
;; oeis.el is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; oeis.el is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You can get a copy of the GNU General Public License online at
;; <http://www.gnu.org/licenses/>.


;;; Commentary:
;;
;; Suppress the large file warning for some files.

;;; Code:

;;;###autoload
(defgroup large-file-warning-suppress nil
  "Large file warning suppress."
  :prefix "large-file-warning-suppress-"
  :group  'files
  :link   '(url-link :tag "large-file-warning-suppress.el home page"
                     "http://user42.tuxfamily.org/large-file-warning-suppress/index.html"))

(defcustom large-file-warning-suppress-list nil
  "List of files for which large file warning is suppressed."
  :group 'large-file-warning-suppress
  :type  '(repeat file))

(defcustom large-file-warning-suppress-functions nil
  "List of functions to test filenames for large file warning suppress.
Each function is called
    (foo FILENAME)
and it should return non-nil if large file warnings should be
suppressed on FILENAME."

  :group 'large-file-warning-suppress
  :type  '(repeat file))

;;-----------------------------------------------------------------------------

(defun large-file-warning-suppress-p (filename)
  "An internal part of large-file-warning-suppress.el.
Return non-nil if large file warning should be suppressed on
FILENAME.

FILENAME is compared to each `large-file-warning-suppress-list'.
The comparison usese `file-truename' so symlinks to the same file
are treated the same.  (Currently hard links are not noticed, but
perhaps that will change.)

FILENAME is passed to each `large-file-warning-suppress-functions'
for its opinion."

  (let ((lst      large-file-warning-suppress-list)
        (truename (file-truename filename))
        suppress)
    (while (and lst (not suppress))
      (setq suppress (equal truename (file-truename (pop lst)))))

    (setq lst large-file-warning-suppress-functions)
    (while (and lst (not suppress))
      (setq suppress (funcall (pop lst) filename)))

    suppress))

(defadvice abort-if-file-too-large (around large-file-warning-suppress
                                           activate)
  "Don't warn when visiting some large files."
  (if (let ((size     (ad-get-arg 0))  ;; args: size op-type filename
            (filename (ad-get-arg 2)))
        (and large-file-warning-threshold size
             (> size large-file-warning-threshold)
             (not (large-file-warning-suppress-p filename))))
      ad-do-it))

;; ;;;###autoload
;; (defadvice abort-if-file-too-large (around oeis-allow-large-names-or-stripped
;;                                            activate)
;;   "Don't warn when visiting large OEIS names and stripped files."
;;   (when (and (let ((size (ad-get-arg 0)))
;;                (and large-file-warning-threshold size
;;                     (> size large-file-warning-threshold)))
;;              (progn
;;                (require 'oeis)
;;                (not (oeis-filename-is-names-or-stripped (ad-get-arg 2)))))
;;     ad-do-it))
;; 
;; (defun oeis-filename-is-names-or-stripped (filename)
;;   "An internal part of oeis.el.
;; Return non-nil if FILENAME is ~/OEIS/names or ~/OEIS/stripped.
;; FILENAME can be in any of the `oeis-local-directories' and can
;; have a .gz extension."
;;   (setq filename (file-truename filename))
;;   (let (found)
;;     (dolist (dir oeis-local-directories)
;;       (dolist (basename '("names" "stripped"))
;;         (dolist (ext '("" ".gz"))
;;           (or found
;;               (setq found
;;                     (equal filename
;;                            (file-truename (expand-file-name
;;                                            (concat basename ext) dir))))))))
;;     found))


(provide 'large-file-warning-suppress)

;;; large-file-warning-suppress.el ends here
